package com.cmeeting;

import javax.mail.*;
import javax.mail.internet.*;
import java.util.Properties;
import java.io.File;

public class EmailSender {

    // 发件人邮箱配置（建议从配置文件读取，此处写死示例）
    private static final String FROM_EMAIL = "binzhai321@163.com";
    private static final String AUTH_CODE = "RXbRu3AdrCxX57ib"; // 授权码
    private static final String SMTP_HOST = "smtp.163.com";
    private static final int SMTP_PORT = 465;

    /**
     * 发送带附件的邮件
     * @param toEmail   收件人邮箱
     * @param filePath  附件本地路径（如 "C:/test.pdf"）
     * @param subject   邮件主题（可选，默认值）
     * @param text      邮件正文（可选，默认值）
     * @return true发送成功，false发送失败
     */
    public static boolean sendEmailWithAttachment(String toEmail, String filePath,
                                                  String subject, String text) {
        // 1. 参数校验
        if (toEmail == null || toEmail.isEmpty() || filePath == null) {
            System.err.println("参数错误：收件人或附件路径为空");
            return false;
        }

        // 2. 设置默认邮件主题和正文
        String mailSubject = (subject != null) ? subject : "";
        String mailText = (text != null) ? text : "请查收附件。";

        // 3. 配置SMTP
        Properties props = new Properties();
        props.put("mail.smtp.host", SMTP_HOST);
        props.put("mail.smtp.port", SMTP_PORT);
        props.put("mail.smtp.auth", "true");
        props.put("mail.smtp.ssl.enable", "true");

        try {
            // 4. 创建会话
            Session session = Session.getInstance(props, new Authenticator() {
                @Override
                protected PasswordAuthentication getPasswordAuthentication() {
                    return new PasswordAuthentication(FROM_EMAIL, AUTH_CODE);
                }
            });

            // 5. 构建邮件内容
            MimeMessage message = new MimeMessage(session);
            message.setFrom(new InternetAddress(FROM_EMAIL));
            message.setRecipient(Message.RecipientType.TO, new InternetAddress(toEmail));
            message.setSubject(mailSubject);

            // 6. 添加正文和附件
            Multipart multipart = new MimeMultipart();

            // 文本正文
            MimeBodyPart textPart = new MimeBodyPart();
            textPart.setText(mailText);
            multipart.addBodyPart(textPart);

            // 附件
            MimeBodyPart attachmentPart = new MimeBodyPart();
            attachmentPart.attachFile(new File(filePath));
            multipart.addBodyPart(attachmentPart);

            message.setContent(multipart);

            // 7. 发送邮件
            Transport.send(message);
            System.out.println("邮件发送成功至: " + toEmail);
            return true;
        } catch (Exception e) {
            System.err.println("邮件发送失败: " + e.getMessage());
            return false;
        }
    }

    /**
     * 简化版调用（仅需收件人和附件路径）
     */
    public static boolean sendEmailWithAttachment(String toEmail, String filePath) {
        return sendEmailWithAttachment(toEmail, filePath, null, null);
    }
}