package com.cmeeting;

import com.azure.core.credential.AccessToken;
import com.azure.core.credential.TokenRequestContext;
import com.azure.identity.ClientSecretCredential;
import com.azure.identity.ClientSecretCredentialBuilder;
import com.microsoft.graph.authentication.IAuthenticationProvider;
import com.microsoft.graph.requests.AttachmentCollectionPage;
import com.microsoft.graph.requests.AttachmentCollectionResponse;
import com.microsoft.graph.requests.GraphServiceClient;
import com.microsoft.graph.models.*;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.util.Base64;
import java.util.LinkedList;

public class OutlookTest {

    // 创建日志记录器
    private static final Logger logger = LoggerFactory.getLogger(OutlookTest.class);

    public static void main(String[] args) throws Exception {
        // Azure AD 应用程序的客户端凭证
        String clientId = "d65aa91b-05f4-42b2-902f-4d82ea5a2b93";
        String clientSecret = "~N98Q~83v6dViQFIofwr0fRn4J5VEZ2tvwOz.bPX";
        String tenantId = "74cc8acf-aacc-4514-9bbb-dc27ce3096bb";
        String scope = "https://graph.microsoft.com/.default";

        // 使用 ClientSecretCredential 获取 token
        ClientSecretCredential clientSecretCredential = new ClientSecretCredentialBuilder()
                .clientId(clientId).tenantId(tenantId).clientSecret(clientSecret).build();
        if (null == scope || null == clientSecretCredential) {
            throw new Exception("Unexpected error");
        }
        // 创建身份验证提供者
        IAuthenticationProvider authProvider = request -> {
            // 创建 TokenRequestContext 对象并指定所需的范围
            TokenRequestContext requestContext = new TokenRequestContext();
            requestContext.addScopes("https://graph.microsoft.com/.default"); // 这里指定需要的范围

            // 获取访问令牌
            String accessToken = clientSecretCredential.getToken(requestContext).block().getToken();
            // SDK 会自动将令牌添加到请求头中，您不需要手动操作 request 对象
            return null;
        };

        // 创建 Graph 客户端
        GraphServiceClient<Request> graphClient = GraphServiceClient
                .builder()
                .authenticationProvider(authProvider)
                .buildClient();


        // 打印访问令牌
        printAccessToken(clientSecretCredential);

      /*  // 启用调试日志
        GraphServiceClient<okhttp3.Request> graphClient = GraphServiceClient
                .builder()
                .authenticationProvider(tokenCredAuthProvider)
                .httpClient(createHttpClient())  // 设置启用调试日志的 HTTP 客户端
                .buildClient();*/
        // 调用 Graph API 发送邮件
        test(graphClient);
    }


    static void printAccessToken(ClientSecretCredential clientSecretCredential) {
        try {
            TokenRequestContext requestContext = new TokenRequestContext();
            requestContext.addScopes("https://graph.microsoft.com/.default");

            // 获取访问令牌
            AccessToken accessToken = clientSecretCredential.getToken(requestContext).block();

            if (accessToken != null && !accessToken.getToken().isEmpty()) {
                System.out.println("Access Token: " + accessToken.getToken());
                System.out.println("Expires On: " + accessToken.getExpiresAt().toEpochSecond());
            } else {
                System.out.println("Failed to retrieve access token.");
            }
        } catch (Exception e) {
            System.err.println("Error while retrieving access token: " + e.getMessage());
        }
    }


    public static void test(GraphServiceClient<okhttp3.Request> graphClient) throws IOException {
        try {
            // 创建邮件对象
            Message message = new Message();
            message.subject = "Meet for lunch?";
            ItemBody body = new ItemBody();
            body.contentType = BodyType.TEXT;
            body.content = "The new cafeteria is open.";
            message.body = body;

            // 设置收件人
            LinkedList<Recipient> toRecipientsList = new LinkedList<>();
            Recipient toRecipients = new Recipient();
            EmailAddress emailAddress = new EmailAddress();
            emailAddress.address = "binzhai321@outlook.com";
            toRecipients.emailAddress = emailAddress;
            toRecipientsList.add(toRecipients);
            message.toRecipients = toRecipientsList;

            // 创建附件
            LinkedList<Attachment> attachmentsList = new LinkedList<>();
            FileAttachment attachments = new FileAttachment();
            attachments.name = "1111.txt";
            attachments.oDataType = "#microsoft.graph.fileAttachment";
            attachments.contentType = "text/plain";
            attachments.contentBytes = Base64.getDecoder().decode("SGVsbG8gV29ybGQh");
            attachmentsList.add(attachments);
            AttachmentCollectionResponse attachmentCollectionResponse = new AttachmentCollectionResponse();
            attachmentCollectionResponse.value = attachmentsList;

            AttachmentCollectionPage attachmentCollectionPage = new AttachmentCollectionPage(attachmentCollectionResponse, null);
            message.attachments = attachmentCollectionPage;

            // 发送邮件
            graphClient.users("binzhai321@outlook.com")
                    .sendMail(UserSendMailParameterSet.newBuilder()
                            .withMessage(message)
                            .withSaveToSentItems(null)
                            .build())
                    .buildRequest()
                    .post();

            System.out.println("Email sent successfully!");
        } catch (Exception e) {
            System.err.println("Error occurred: " + e.getMessage());
            if (e instanceof com.microsoft.graph.http.GraphServiceException) {
                com.microsoft.graph.http.GraphServiceException graphException = (com.microsoft.graph.http.GraphServiceException) e;
                System.err.println("Error code: " + graphException.getServiceError().code);
                System.err.println("Error message: " + graphException.getServiceError().message);
            }
            e.printStackTrace();
        }
    }


    private static OkHttpClient createHttpClient() {
        return new OkHttpClient.Builder()
                .addInterceptor(chain -> {
                    okhttp3.Request originalRequest = chain.request();

                    // 打印原始请求头
                    System.out.println("Original Request Headers: " + originalRequest.headers());

                    // 确保 Authorization 头未被覆盖
                    okhttp3.Request modifiedRequest = originalRequest.newBuilder()
                            .headers(originalRequest.headers())  // 保留原始请求头
                            .build();

                    return chain.proceed(modifiedRequest);
                })
                .build();
    }
}
